@@ -33,8 +33,10 @@
 /* 32 bit, 4 values */
 #define WACOM_PROP_PRESSURECURVE "Wacom Pressurecurve"
 
-/* CARD32, 4 values, tablet id, old serial, old hw device id, current serial
-   read-only */
+/* CARD32, 5 values, tablet id, old serial, old hw device id,
+   current serial, current device id
+   read-only
+ */
 #define WACOM_PROP_SERIALIDS "Wacom Serial IDs"
 
 /* CARD32, 1 value */
@@ -70,7 +72,7 @@
 /* 32 bit, 1 value */
 #define WACOM_PROP_PRESSURE_THRESHOLD "Wacom Pressure Threshold"
 
-/* 32 bit, 2 values, sample, suppress */
+/* 32 bit, 2 values, suppress, sample */
 #define WACOM_PROP_SAMPLE "Wacom Sample and Suppress"
 
 /* BOOL, 1 value */
