@@ -320,12 +320,9 @@ static int wcmFindProp(Atom property, Atom *prop_list, int nprops)
 	/* check all properties used for button actions */
 	for (i = 0; i < nprops; i++)
 		if (prop_list[i] == property)
-			break;
+			return i;
 
-	if (i >= nprops)
-		return -BadAtom;
-
-	return i;
+	return -BadAtom;
 }
 
 /**
@@ -337,8 +334,10 @@ static int wcmFindProp(Atom property, Atom *prop_list, int nprops)
  * @param property      The Action property that should be searched for
  * @param[out] handler  Returns a pointer to the property's handler
  * @param[out] action   Returns a pointer to the property's action list
+ * @return              'true' if the property was found. Neither out parameter
+ *                      will be null if this is the case.
  */
-static void wcmFindActionHandler(WacomDevicePtr priv, Atom property, Atom **handler, unsigned int (**action)[256])
+static BOOL wcmFindActionHandler(WacomDevicePtr priv, Atom property, Atom **handler, unsigned int (**action)[256])
 {
 	int offset;
 
@@ -347,7 +346,7 @@ static void wcmFindActionHandler(WacomDevicePtr priv, Atom property, Atom **hand
 	{
 		*handler = &priv->btn_actions[offset];
 		*action  = &priv->keys[offset];
-		return;
+		return TRUE;
 	}
 
 	offset = wcmFindProp(property, priv->wheel_actions, ARRAY_SIZE(priv->wheel_actions));
@@ -355,28 +354,49 @@ static void wcmFindActionHandler(WacomDevicePtr priv, Atom property, Atom **hand
 	{
 		*handler = &priv->wheel_actions[offset];
 		*action  = &priv->wheel_keys[offset];
-		return;
+		return TRUE;
 	}
 
 	offset = wcmFindProp(property, priv->strip_actions, ARRAY_SIZE(priv->strip_actions));
-	if (offset < 0)
+	if (offset >= 0)
 	{
 		*handler = &priv->strip_actions[offset];
 		*action  = &priv->strip_keys[offset];
-		return;
+		return TRUE;
 	}
+
+	return FALSE;
 }
 
-static int wcmCheckActionProperty(Atom property, XIPropertyValuePtr prop)
+static int wcmCheckActionProperty(WacomDevicePtr priv, Atom property, XIPropertyValuePtr prop)
 {
 	CARD32 *data;
 	int j;
 
-	if (!property)
-		return Success;
-
-	if (prop->size >= 255 || prop->format != 32 || prop->type != XA_INTEGER)
+	if (!property) {
+		DBG(3, priv, "ERROR: Atom is NONE\n");
 		return BadMatch;
+	}
+
+	if (prop == NULL) {
+		DBG(3, priv, "ERROR: Value is NULL\n");
+		return BadMatch;
+	}
+
+	if (prop->size >= 255) {
+		DBG(3, priv, "ERROR: Too many values (%ld > 255)\n", prop->size);
+		return BadMatch;
+	}
+
+	if (prop->format != 32) {
+		DBG(3, priv, "ERROR: Incorrect value format (%d != 32)\n", prop->format);
+		return BadMatch;
+	}
+
+	if (prop->type != XA_INTEGER) {
+		DBG(3, priv, "ERROR: Incorrect value type (%d != XA_INTEGER)\n", prop->type);
+		return BadMatch;
+	}
 
 	data = (CARD32*)prop->data;
 
@@ -385,19 +405,25 @@ static int wcmCheckActionProperty(Atom property, XIPropertyValuePtr prop)
 		int code = data[j] & AC_CODE;
 		int type = data[j] & AC_TYPE;
 
+		DBG(10, priv, "Index %d == %d (type: %d, code: %d)\n", j, data[j], type, code);
+
 		switch(type)
 		{
 			case AC_KEY:
 				break;
 			case AC_BUTTON:
-				if (code > WCM_MAX_BUTTONS)
+				if (code > WCM_MAX_X11BUTTON) {
+					DBG(3, priv, "ERROR: AC_BUTTON code too high (%d > %d)\n", code, WCM_MAX_X11BUTTON);
 					return BadValue;
+				}
 				break;
 			case AC_DISPLAYTOGGLE:
 			case AC_MODETOGGLE:
 				break;
 			default:
+				DBG(3, priv, "ERROR: Unknown command\n");
 				return BadValue;
+				break;
 		}
 	}
 
@@ -425,15 +451,23 @@ static int wcmSetActionProperty(DeviceIntPtr dev, Atom property,
 	WacomDevicePtr priv = (WacomDevicePtr) pInfo->private;
 	int rc, i;
 
-	DBG(10, priv, "\n");
+	DBG(5, priv, "%s new actions for Atom %d\n", checkonly ? "Checking" : "Setting", property);
 
-	rc = wcmCheckActionProperty(property, prop);
-	if (rc != Success)
+	rc = wcmCheckActionProperty(priv, property, prop);
+	if (rc != Success) {
+		const char *msg = NULL;
+		switch (rc) {
+			case BadMatch: msg = "BadMatch"; break;
+			case BadValue: msg = "BadValue"; break;
+			default: msg = "UNKNOWN"; break;
+		}
+		DBG(3, priv, "Action validation failed with code %d (%s)\n", rc, msg);
 		return rc;
+	}
 
-	if (!checkonly && prop)
+	if (!checkonly)
 	{
-		memset(action, 0, sizeof(action));
+		memset(action, 0, sizeof(*action));
 		for (i = 0; i < prop->size; i++)
 			(*action)[i] = ((unsigned int*)prop->data)[i];
 		*handler = property;
@@ -485,7 +519,10 @@ static int wcmSetActionsProperty(DeviceIntPtr dev, Atom property,
                                  int size, Atom* handlers, unsigned int (*actions)[256])
 {
 	InputInfoPtr pInfo = (InputInfoPtr) dev->public.devicePrivate;
-	int rc;
+	WacomDevicePtr priv = (WacomDevicePtr) pInfo->private;
+	int rc, i;
+
+	DBG(10, priv, "\n");
 
 	if (prop->size != size)
 		return BadValue;
@@ -494,28 +531,26 @@ static int wcmSetActionsProperty(DeviceIntPtr dev, Atom property,
 	if (rc != Success)
 		return rc;
 
-	if (!checkonly)
+	for (i = 0; i < prop->size; i++)
 	{
-		int i;
+		int index = i;
+		Atom subproperty = ((Atom*)prop->data)[i];
+		XIPropertyValuePtr subprop;
 
-		for (i = 0; i < prop->size; i++)
-		{
-			int index = i;
-			Atom subproperty = ((Atom*)prop->data)[i];
-			XIPropertyValuePtr subprop;
+		if (property == prop_btnactions)
+		{ /* Driver uses physical -- not X11 -- button numbering internally */
+			if (i < 3)
+				index = i;
+			else if (i < 7)
+				continue;
+			else
+				index = i - 4;
+		}
 
-			if (property == prop_btnactions)
-			{ /* Driver uses physical -- not X11 -- button numbering internally */
-				if (i < 3)
-					index = i;
-				else if (i < 7)
-					continue;
-				else
-					index = i - 4;
-			}
-
-			if (subproperty == 0)
-			{ /* Interpret 'None' as meaning 'reset' */
+		if (subproperty == 0)
+		{ /* Interpret 'None' as meaning 'reset' */
+			if (!checkonly)
+			{
 				if (property == prop_btnactions)
 					wcmResetButtonAction(pInfo, index, size);
 				else if (property == prop_strip_buttons)
@@ -526,9 +561,13 @@ static int wcmSetActionsProperty(DeviceIntPtr dev, Atom property,
 				if (subproperty != handlers[index])
 					subproperty = handlers[index];
 			}
-
+		}
+		else
+		{
 			XIGetDeviceProperty(dev, subproperty, &subprop);
-			wcmSetActionProperty(dev, subproperty, subprop, checkonly, &handlers[index], &actions[index]);
+			rc = wcmSetActionProperty(dev, subproperty, subprop, checkonly, &handlers[index], &actions[index]);
+			if (rc != Success)
+				return rc;
 		}
 	}
 
@@ -813,9 +852,8 @@ int wcmSetProperty(DeviceIntPtr dev, Atom property, XIPropertyValuePtr prop,
 	{
 		Atom *handler = NULL;
 		unsigned int (*action)[256] = NULL;
-		wcmFindActionHandler(priv, property, &handler, &action);
-		if (handler != NULL && action != NULL)
-			wcmSetActionProperty(dev, property, prop, checkonly, handler, action);
+		if (wcmFindActionHandler(priv, property, &handler, &action))
+			return wcmSetActionProperty(dev, property, prop, checkonly, handler, action);
 		/* backwards-compatible behavior silently ignores the not-found case */
 	}
 
@@ -925,7 +963,7 @@ wcmUpdateSerial(InputInfoPtr pInfo, unsigned int serial, int id)
 {
 	WacomDevicePtr priv = pInfo->private;
 
-	if (priv->cur_serial == serial)
+	if (priv->cur_serial == serial && priv->cur_device_id == id)
 		return;
 
 	priv->cur_serial = serial;
