@@ -296,20 +296,21 @@ static int wcmInitAxes(DeviceIntPtr pWcm)
 
 
 	/* seventh valuator: abswheel2 */
-	index = 6;
-	label = None;
-	mode = Absolute;
-	min_res = max_res = res = 1;
-	min = 0;
-	max = 1;
-
 	if ((TabletHasFeature(common, WCM_DUALRING)) && IsPad(priv))
-	{ /* XXX: what is this axis label? */
+	{
+		/* XXX: what is this axis label? */
+		index = 6;
+		label = None;
+		mode = Absolute;
+		min_res = max_res = res = 1;
+		min = 0;
+		max = 1;
+
 		min = MIN_PAD_RING;
 		max = MAX_PAD_RING;
-	}
 
-	wcmInitAxis(pInfo->dev, index, label, min, max, res, min_res, max_res, mode);
+		wcmInitAxis(pInfo->dev, index, label, min, max, res, min_res, max_res, mode);
+	}
 
 	return TRUE;
 }
@@ -483,7 +484,7 @@ Bool wcmIsWacomDevice (char* fname)
  ****************************************************************************/
 #define DEV_INPUT_EVENT "/dev/input/event%d"
 #define EVDEV_MINORS    32
-const char *wcmEventAutoDevProbe (InputInfoPtr pInfo)
+char *wcmEventAutoDevProbe (InputInfoPtr pInfo)
 {
 	/* We are trying to find the right eventX device */
 	int i, wait = 0;
@@ -506,7 +507,7 @@ const char *wcmEventAutoDevProbe (InputInfoPtr pInfo)
 				xf86ReplaceStrOption(pInfo->options, "Device", fname);
 
 				/* this assumes there is only one Wacom device on the system */
-				return xf86FindOptionValue(pInfo->options, "Device");
+				return xf86CheckStrOption(pInfo->options, "Device", NULL);
 			}
 		}
 		wait += 100;
