@@ -394,7 +394,6 @@ extern WacomDeviceClass gWacomISDV4Device;
  * WacomCommonRec
  *****************************************************************************/
 
-#define TILT_REQUEST_FLAG       1
 #define TILT_ENABLED_FLAG       2
 
 #define MAX_FINGERS 16
@@ -419,7 +418,7 @@ enum WacomProtocol {
 struct _WacomCommonRec 
 {
 	/* Do not move device_path, same offset as priv->name. Used by DBG macro */
-	const char* device_path;    /* device file name */
+	char* device_path;          /* device file name */
 	dev_t min_maj;               /* minor/major number */
 	unsigned char wcmFlags;     /* various flags (handle tilt) */
 	int debugLevel;
